@@ -431,6 +431,7 @@ class Local(object):
     def get_changes(self, install_root='/'):
         '''Scan the build directories for files that are different than what's in the install root, using sha1 hashes to compare file contents'''
         changed = []
+        self.merge_partials()
 
         def file_hash(path):
             h = sha()
@@ -469,43 +470,6 @@ class Local(object):
 
         return changed
 
-    def snapshot(self, root='/', roles=None):
-        '''Save a snapshot of the current node to the repository filesystem for diff to make sense of later'''
-        if roles is None:
-            roles = self.roles
-        packages = package.Package().list()
-        files = {}
-        for role in roles:
-            self.snapshot_files(role, files, root)
-
-        f = open(os.path.join(self.path, '.snapshot'), 'w')
-        pickle.dump({'packages': packages, 'files': files}, f)
-        f.close()
-
-    def snapshot_files(self, role, files, root):
-        '''Accept a list of roles and an accumulator, and fill the accumulator with file data'''
-        role = self.role(role)
-        for path in [os.path.join(root, path) for path in role.files if os.path.exists(os.path.join(root, path)) and os.path.join(root, path) not in files]:
-            files[path] = {'type': 'files', 'mtime': os.stat(path).st_mtime, 'owner': role.name}
-
-        for path in [os.path.join(root, path) for path in role.templates if os.path.exists(os.path.join(root, path)) and os.path.join(root, path) not in files]:
-            files[path] = {'type': 'templates', 'mtime': os.stat(path).st_mtime, 'owner': role.name}
-
-        for mixin in role.mixins:
-            self.snapshot_files(mixin, files, root)
-
-    
-    def diff(self, roles, root='/'):
-        '''Compare the installed packages and role-handled files to the last snapshot'''
-        snapshot = pickle.load(open(os.path.join(self.path, '.snapshot'), 'r'))
-        new_packages = [p for p in package.Package().list() if p not in snapshot['packages']]
-
-        new_files = {}
-        for path, info in snapshot['files'].iteritems():
-            if os.stat(path).st_mtime > info['mtime']:
-                new_files[path] = info
-        return new_packages, new_files
-
     def extract_package(self, package_name, role_name):
         '''Extract the config files a given uninstalled package into a given role'''
         role_path = os.path.join(util.get_repopath(), role_name)
@@ -528,11 +492,53 @@ class Local(object):
         fcntl.flock(self._locked, fcntl.LOCK_EX)
         return True
 
+    def merge_partials(self):
+        '''Scan the build directory for any defined partials and merge them into files'''
+        meta_path = os.path.join(self.build_path, '.bishop')
+        if sys.version_info >= (2,6):
+            files_to_walk = os.walk(meta_path, followlinks=True)
+        else:
+            files_to_walk = os.walk(meta_path)
+
+        for root, dirs, files in files_to_walk:
+            rel_root = root.replace(meta_path, '')
+            if rel_root.startswith('/'):
+                rel_root = rel_root[1:]
+
+            partials = {}
+            for file in [f for f in files if '-bpartial-' in f]:
+                file_path = os.path.join(root, file)
+                name = os.path.join(rel_root, file.split('-bpartial-')[0])
+                if name in partials:
+                    partials[name].append(file_path)
+                else:
+                    partials[name] = [file_path]
+            
+            if len(partials) > 0:
+                for name, paths in partials.items():
+                    paths.sort()
+                    partial_content = '\n'.join([open(path).read() for path in paths])
+                    build_path = os.path.join(self.build_path, name)
+                    if not os.path.exists(build_path):
+                        try:
+                            os.makedirs(os.path.dirname(build_path))
+                        except OSError:
+                            pass
+                        open(build_path, 'w').write(partial_content)
+                    else:
+                        content = open(build_path).read()
+                        os.remove(build_path)
+                        open(build_path, 'w').write(content + '\n' + partial_content)
+
     def unlock(self):
         '''Unlock a given lock'''
+        if not self._locked:
+            return False
+
         fcntl.flock(self._locked, fcntl.LOCK_UN)
         self._locked.close()
         self._locked = False
+        return True
 
 class Remote(object):
     def __init__(self, host):
